@@ -89,6 +89,15 @@ class baseDAO
      */
     public $slaveDBH;
 
+    /**
+     * 全局对象$cache。
+     * The global cache object.
+     *
+     * @var object
+     * @access public
+     */
+    public $cache = null;
+
     /**
      * sql对象，用于生成sql语句。
      * The sql object, used to create the query sql.
@@ -172,6 +181,15 @@ class baseDAO
      */
     static public $querys = array();
 
+    /**
+     * 执行fetchAll是否跳过text类型字段。
+     * Exclude text fields when fetchAll.
+     *
+     * @var bool
+     * @access public
+     */
+    static public $autoExclude = false;
+
     /**
      * 存放错误的数组。
      * The errors.
@@ -200,6 +218,15 @@ class baseDAO
      */
     public static $tablesDesc = array();
 
+    /**
+     * 缓存已经查询过的唯一索引。
+     * Cache unique indexes.
+     *
+     * @var    array
+     * @access private
+     */
+    protected static $uniqueIndexes = [];
+
     /**
      * 构造方法。
      * The construct method.
@@ -211,24 +238,12 @@ class baseDAO
     public function __construct($app)
     {
         global $config, $lang, $dbh, $slaveDBH;
-        $this->app       = $app;
-        $this->config    = $config;
-        $this->lang      = $lang;
-        $this->dbh       = $dbh;
-        $this->slaveDBH  = $slaveDBH ? $slaveDBH : false;
-
-        if($config->cache->dao->enable)
-        {
-            $this->app->loadClass('cache', true);
-            try
-            {
-                $this->cache = new cache($config->cache->dao->driver, $config->db->name . '-dao-', $config->cache->dao->lifetime);
-            }
-            catch (Exception $e)
-            {
-                die($e->getMessage());
-            }
-        }
+        $this->app      = $app;
+        $this->config   = $config;
+        $this->lang     = $lang;
+        $this->dbh      = $dbh;
+        $this->cache    = $app->cache;
+        $this->slaveDBH = $slaveDBH ? $slaveDBH : false;
 
         $this->reset();
     }
@@ -331,29 +346,44 @@ class baseDAO
         $this->method = $method;
     }
 
+    /**
+     * 生成缓存的 key。
+     * Create the cache key.
+     *
+     * @param  mixed  $args
+     * @access private
+     * @return string
+     */
+    private function createCacheKey(...$args)
+    {
+        if(empty($this->cache)) return implode('-', $args);
+
+        return $this->cache->createKey('dao', ...$args);
+    }
+
     /**
      * 获取缓存。
      * Get the cache.
      *
      * @param  string $key
-     * @param  string $sql
      * @access public
      * @return mixed
      */
-    public function getCache($key, $sql)
+    public function getCache($key)
     {
         if(!$this->app->isServing() || empty($this->cache)) return self::CACHE_MISS;
 
-        $cache = $this->cache->get($key);
-        if($cache === null) return self::CACHE_MISS;
+        $cache = $this->cache->getByKey($key);
+        if($cache === null)   return self::CACHE_MISS;
+        if(count($cache) < 3) return self::CACHE_MISS;
 
         /* 解析缓存的更新时间和值到变量中。 */
         /* Parse the cache time and value to variables. */
-        list($cachedTime, $cachedValue) = $cache;
+        list($cachedTime, $cachedSQL, $cachedValue) = $cache;
 
         /* 查找 sql 语句中包含的表名。*/
         /* Find the table names in the sql. */
-        preg_match_all("/({$this->config->db->prefix}\w+)[`\" ]/", $sql, $tables);
+        preg_match_all("/({$this->config->db->prefix}\w+)[`\" ]/", $cachedSQL, $tables);
         if(!isset($tables[1])) return self::CACHE_MISS;
 
         /* 检查 sql 语句中包含的表的更新时间是否大于缓存的更新时间，如果大于则不使用缓存。*/
@@ -362,7 +392,8 @@ class baseDAO
         {
             if(strpos($table, 'boardlayer') !== false) return self::CACHE_MISS;
 
-            $tableCache = $this->cache->get($table);
+            $tableKey   = $this->createCacheKey('table', $table);
+            $tableCache = $this->cache->getByKey($tableKey);
             if($tableCache === null) continue;
 
             if($tableCache[0] > $cachedTime) return self::CACHE_MISS;
@@ -372,7 +403,7 @@ class baseDAO
         /* Check if can use the client cache. */
         $this->app->useClientCache = $this->app->clientCacheTime > $cachedTime;
 
-        return $cachedValue;
+        return $cachedValue ?: self::CACHE_MISS;
     }
 
     /**
@@ -380,17 +411,19 @@ class baseDAO
      * Set the cache.
      *
      * @param  string $key
+     * @param  string $sql
      * @param  mixed  $value
+     * @param  int    $ttl
      * @access public
      * @return void
      */
-    public function setCache($key, $value = null)
+    public function setCache($key, $sql = '', $value = null, $ttl = null)
     {
         if(!$this->app->isServing() || empty($this->cache)) return false;
 
         $this->app->useClientCache = false;
 
-        $this->cache->set($key, array(microtime(true), $value));
+        $this->cache->saveByKey($key, array(microtime(true), $sql, $value), $ttl ?? $this->config->cache->dao->lifetime);
     }
 
     /**
@@ -413,7 +446,8 @@ class baseDAO
             /* 更新表的缓存时间。*/
             /* Update the table cache time. */
             $table = str_replace(array('`', '"'), '', $table);
-            $this->setCache($table);
+            $key   = $this->createCacheKey('table', $table);
+            $this->setCache($key, '', $table, 0);
         }
     }
 
@@ -775,11 +809,11 @@ class baseDAO
     public function explain($sql = '', $exit = true)
     {
         $sql    = empty($sql) ? $this->processSQL() : $sql;
-        $result = $this->dbh->rawQuery('explain ' . $sql)->fetch();
+        $result = $this->dbh->rawQuery('explain ' . $sql)->fetchAll();
 
         if($exit) a($result);
 
-        return (array)$result;
+        return $result;
     }
 
     /**
@@ -1018,6 +1052,81 @@ class baseDAO
         return $this;
     }
 
+    /**
+     * 获取唯一索引。
+     * Get unique indexes.
+     *
+     * @param  string $table
+     * @access public
+     * @return array
+     */
+    private function getUniqueIndexes($table)
+    {
+        if(isset(dao::$uniqueIndexes[$table])) return dao::$uniqueIndexes[$table];
+
+        $indexes = [];
+        $table   = trim($table, '`');
+        $rows    = $this->select('INDEX_NAME, COLUMN_NAME')->from('INFORMATION_SCHEMA.STATISTICS')->where('TABLE_SCHEMA')->eq($this->config->db->name)->andWhere('TABLE_NAME')->eq($table)->andWhere('NON_UNIQUE')->eq(0)->andWhere('INDEX_NAME')->ne('PRIMARY')->query()->fetchAll();
+        foreach($rows as $row) $indexes[$row->INDEX_NAME][] = $row->COLUMN_NAME;
+
+        dao::$uniqueIndexes[$table] = $indexes;
+
+        return $indexes;
+    }
+
+    /**
+     * 把 replace 转换为 delete 和 insert。
+     * Convert replace to delete and insert.
+     *
+     * @param  string $table
+     * @access private
+     * @return int
+     */
+    private function convertReplaceToInsert($table)
+    {
+        $processedData = new stdclass();
+        foreach($this->sqlobj->data as $field => $value)
+        {
+            $field = trim($field, '`');
+            $processedData->{$field} = $value;
+        }
+
+        $indexes = $this->getUniqueIndexes($table);
+        if(!$indexes)
+        {
+            dao::$errors[] = "The table {$table} has no unique indexes.";
+            return 0;
+        }
+
+        $inTransaction = $this->inTransaction();
+        if(!$inTransaction) $this->begin();
+
+        foreach($indexes as $fields)
+        {
+            $this->delete()->from($table)->where('1=1');
+            foreach($fields as $field)
+            {
+                if(!isset($processedData->$field))
+                {
+                    dao::$errors[] = "The field $field of table {$table} is required.";
+                    return 0;
+                }
+                $this->andWhere("`{$field}`")->eq($processedData->$field);
+            }
+            $this->exec();
+        }
+
+        $result = $this->insert($table)->data($processedData)->exec();
+
+        if(!$inTransaction)
+        {
+            if(!$result) $this->rollback();
+            $this->commit();
+        }
+
+        return $result;
+    }
+
     /**
      * 执行SQL。query()会返回stmt对象，该方法只返回更改或删除的记录数。
      * Execute the sql. It's different with query(), which return the stmt object. But this not.
@@ -1028,7 +1137,15 @@ class baseDAO
      */
     public function exec($sql = '')
     {
-        if(!empty(dao::$errors)) return 0;
+        if(dao::isError()) return 0;
+
+        if($this->method == 'replace' && !empty($this->sqlobj->data))
+        {
+            $table = $this->table;
+            if(strpos($table, '`') === false) $table = "`{$table}`";
+
+            if(isset($this->config->cache->raw[$table])) return $this->convertReplaceToInsert($table);
+        }
 
         if($sql)
         {
@@ -1047,26 +1164,42 @@ class baseDAO
             /* Real-time save log. */
             if(dao::$realTimeLog && dao::$realTimeFile) file_put_contents(dao::$realTimeFile, $sql . "\n", FILE_APPEND);
 
-            $table = trim($this->table, '`');
+            $table  = $this->table;
+            $method = $this->method;
             $this->reset();
 
             /* Force to query from master db, if db has been changed. */
             $this->slaveDBH = false;
 
-            $result = $this->dbh->exec($sql);
-            /* See: https://www.php.net/manual/en/pdo.lastinsertid.php .*/
-            $this->_lastInsertID = $this->dbh->lastInsertId();
+            if($this->cache) $this->cache->prepare($table, $method, $sql);
 
-            $this->setTableCache($sql);
+            $result = $this->dbh->exec($sql);
+
+            /* See: https://www.php.net/manual/en/pdo.lastinsertid.php .*/
+            if($method == 'insert') $this->_lastInsertID = $this->dbh->lastInsertID();
+
+            if($this->cache)
+            {
+                if($result)
+                {
+                    $this->setTableCache($sql);
+                    $this->cache->sync();
+                }
+                else
+                {
+                    $this->cache->reset();
+                }
+            }
 
             if($this->config->enableDuckdb)
             {
-                $now        = helper::now();
-                $queueTable = trim(TABLE_DUCKDBQUEUE, '`');
+                $queueTable = TABLE_DUCKDBQUEUE;
                 if(!empty($table) && $table != $queueTable)
                 {
-                    $this->dbh->exec("UPDATE {$queueTable} SET updatedTime = '$now' WHERE object = '$table'");
-                    $this->dbh->exec("INSERT INTO {$queueTable} (object, updatedTime, syncTime) SELECT '$table', '$now', NULL WHERE NOT EXISTS (SELECT 1 FROM {$queueTable} WHERE object = '$table' );");
+                    $now    = helper::now();
+                    $object = trim($table, '`');
+                    $this->dbh->exec("UPDATE {$queueTable} SET updatedTime = '$now' WHERE object = '$object'");
+                    $this->dbh->exec("INSERT INTO {$queueTable} (`object`, `updatedTime`, `syncTime`) SELECT '$object', '$now', NULL WHERE NOT EXISTS (SELECT 1 FROM {$queueTable} WHERE `object` = '$object' );");
                 }
             }
 
@@ -1091,14 +1224,13 @@ class baseDAO
      */
     public function fetch($field = '')
     {
-        $sql = $this->processSQL();
-        $key = 'fetch-' . md5($sql);
-
-        $result = $this->getCache($key, $sql);
+        $sql    = $this->processSQL();
+        $key    = $this->createCacheKey('fetch', md5($sql));
+        $result = $this->getCache($key);
         if($result === self::CACHE_MISS)
         {
             $result = $this->query($sql)->fetch(PDO::FETCH_OBJ);
-            $this->setCache($key, $result);
+            $this->setCache($key, $sql, $result);
         }
 
         if(empty($field)) return $result;
@@ -1106,25 +1238,85 @@ class baseDAO
         return $result ? $result->$field : '';
     }
 
+    /**
+     * 匹配SQL语句中的表别名，返回['t1.*' => 't1', '*' => '']
+     * Match table alias name.
+     *
+     * @param  string $sql
+     * @access private
+     * @return array
+     */
+    private function matchTableAlias($sql)
+    {
+        $pattern = '/SELECT\s+((\w+\.\*,?\s*)+|\*)/i';
+
+        if(preg_match($pattern, $sql, $matches))
+        {
+            if (trim($matches[1]) === '*') return ['*' => ''];
+
+            /* Get table alias name */
+            preg_match_all('/(\w+)\.\*/', $matches[1], $aliasName);
+
+            return !empty($aliasName[1]) ? [$aliasName[0][0] => $aliasName[1][0]] : [];
+        }
+
+        return [];
+    }
+
+    /**
+     * 将SQL语句中的*展开为具体的字段。
+     * Extract fields in SQL.
+     *
+     * @param  string $sql
+     * @access private
+     * @return string
+     */
+    private function extractSQLFields($sql)
+    {
+        $aliasList = $this->matchTableAlias($sql);
+
+        foreach($aliasList as $selectStr => $tableAlias)
+        {
+            /* Get fields for selectStr. */
+            $tableName = $this->sqlobj->tableAlias[$tableAlias];
+            $fields    = $this->descTable($tableName);
+
+            /* 使用具体的字段替换星号。 Replace selectStr with fields. */
+            $tableFields = [];
+            foreach($fields as $field)
+            {
+                if(strpos($field->type, 'text') !== false || strpos($field->type, 'blob') !== false) continue;
+
+                $tableFields[] = ($tableAlias ? $tableAlias . '.`' : '`') . $field->field . '`';
+            }
+
+            $sql = str_replace($selectStr, implode(',', $tableFields), $sql);
+        }
+
+        return $sql;
+    }
+
     /**
      * 获取所有记录。
      * Fetch all records.
      *
      * @param  string $keyField     返回以该字段做键的记录
      *                              the key field, thus the return records is keyed by this field
+     * @param  bool   $autoExclude  是否排除text类型字段 exclude field type of text
      * @access public
      * @return array the records
      */
-    public function fetchAll($keyField = '')
+    public function fetchAll($keyField = '', $autoExclude = true)
     {
         $sql = $this->processSQL();
-        $key = 'fetchAll-' . md5($sql);
+        if(self::$autoExclude && $autoExclude) $sql = $this->extractSQLFields($sql);
 
-        $rows = $this->getCache($key, $sql);
+        $key  = $this->createCacheKey('fetchAll', md5($sql));
+        $rows = $this->getCache($key);
         if($rows === self::CACHE_MISS)
         {
             $rows = $this->query($sql)->fetchAll();
-            $this->setCache($key, $rows);
+            $this->setCache($key, $sql, $rows);
         }
 
         if(empty($keyField)) return $rows;
@@ -1145,15 +1337,13 @@ class baseDAO
      */
     public function fetchGroup($groupField, $keyField = '')
     {
-        $sql   = $this->processSQL();
-        $table = $this->table;
-        $key   = 'fetchGroup-' . md5($sql);
-
-        $rows = $this->getCache($key, $sql);
+        $sql = $this->processSQL();
+        $key = $this->createCacheKey('fetchAll', md5($sql));
+        $rows = $this->getCache($key);
         if($rows === self::CACHE_MISS)
         {
             $rows = $this->query($sql)->fetchAll();
-            $this->setCache($key, $rows);
+            $this->setCache($key, $sql, $rows);
         }
 
         $result = array();
@@ -1178,14 +1368,13 @@ class baseDAO
      */
     public function fetchPairs($keyField = '', $valueField = '')
     {
-        $sql = $this->processSQL();
-        $key = 'fetchPairs-' . md5($sql);
-
-        $rows = $this->getCache($key, $sql);
+        $sql  = $this->processSQL();
+        $key  = $this->createCacheKey('fetchAll', md5($sql));
+        $rows = $this->getCache($key);
         if($rows === self::CACHE_MISS)
         {
             $rows = $this->query($sql)->fetchAll();
-            $this->setCache($key, $rows);
+            $this->setCache($key, $sql, $rows);
         }
 
         $ready      = false;
@@ -1582,19 +1771,8 @@ class baseDAO
      */
     public function getFieldsType()
     {
-        try
-        {
-            $this->dbh->setAttribute(PDO::ATTR_CASE, PDO::CASE_LOWER);
-            $sql = "DESC $this->table";
-            $rawFields = $this->dbh->rawQuery($sql)->fetchAll();
-            $this->dbh->setAttribute(PDO::ATTR_CASE, PDO::CASE_NATURAL);
-        }
-        catch (PDOException $e)
-        {
-            $this->sqlError($e);
-        }
-
-        $fields = array();
+        $fields    = array();
+        $rawFields = $this->descTable($this->table);
         foreach($rawFields as $rawField)
         {
             $firstPOS = strpos($rawField->type, '(');
@@ -1667,10 +1845,82 @@ class baseDAO
          */
         if($this->app->throwError)
         {
-            return throw new Exception($message);
+            throw new Exception($message);
         }
         $this->app->triggerError($message, __FILE__, __LINE__, $exit = true);
     }
+
+    /**
+     * 获取本次会话的 SQL 语句和执行时间。
+     * Get SQL statements and execution time of current session.
+     *
+     * @access public
+     * @return array
+     */
+    public function getProfiles()
+    {
+        $profiles          = [];
+        $performanceSchema = $this->query("SHOW VARIABLES LIKE 'performance_schema'")->fetch();
+        $performanceSwitch = $performanceSchema->Value ?? 'OFF';
+        if($performanceSwitch == 'ON')
+        {
+            try
+            {
+                $sql      = "SELECT t1.EVENT_ID AS Query_ID, TRUNCATE(t1.TIMER_WAIT/1000000000000,6) AS Duration, t1.SQL_TEXT AS Query FROM performance_schema.events_statements_history_long AS t1 LEFT JOIN performance_schema.threads AS t2 ON t1.THREAD_ID=t2.THREAD_ID wHeRe t2.PROCESSLIST_ID=CONNECTION_ID() oRdEr bY EVENT_ID";
+                $profiles = $this->dbh->query($sql)->fetchAll();
+            }
+            catch(PDOException $e)
+            {
+                $profiles = $this->query('SHOW PROFILES')->fetchAll();
+            }
+        }
+        if(empty($profiles)) $profiles = $this->query('SHOW PROFILES')->fetchAll();
+
+        foreach($profiles as $profile)
+        {
+            $profile->Duration = round((float)$profile->Duration, 4);
+            $profile->Explain  = [];
+            $profile->Error    = '';
+
+            $sql = trim($profile->Query);
+            if(stripos($sql, 'select') !== 0
+                && strpos($sql, 'insert') !== 0
+                && strpos($sql, 'update') !== 0
+                && strpos($sql, 'delete') !== 0
+                && strpos($sql, 'replace') !== 0
+            )
+            {
+                continue;
+            }
+
+            try
+            {
+                $slow = false;
+                $rows = $this->explain($sql, false);
+                foreach($rows as $row)
+                {
+                    if($row->type === 'ALL'
+                        || stripos($row->Extra, 'temporary') !== false
+                        || stripos($row->Extra, 'filesort') !== false
+                        || stripos($row->Extra, 'join buffer') !== false
+                        || stripos($row->Extra, 'checked for each record') !== false
+                        || stripos($row->Extra, 'full scan on null key') !== false
+                    )
+                    {
+                        $slow = true;
+                        break;
+                    }
+                }
+                if($slow) $profile->Explain = $rows;
+            }
+            catch(PDOException $e)
+            {
+                $profile->Error = 'Can not explain the sql statement.';
+            }
+        }
+
+        return $profiles;
+    }
 }
 
 /**
@@ -1805,6 +2055,24 @@ class baseSQL
      */
     public $magicQuote;
 
+    /**
+     * 表别名。
+     * Table alias.
+     *
+     * @var array
+     * @access public
+     */
+    public $tableAlias;
+
+    /**
+     * 当前操作的表。
+     * Current table.
+     *
+     * @var array
+     * @access public
+     */
+    public $currentTable;
+
     /**
      * 构造方法。
      * The construct function.
@@ -1818,6 +2086,7 @@ class baseSQL
         $this->dbh        = $dbh;
         $this->data       = new stdclass();
         $this->skipFields = '';
+        $this->tableAlias = [];
         $this->magicQuote = (version_compare(phpversion(), '5.4', '<') and function_exists('get_magic_quotes_gpc') and get_magic_quotes_gpc());
     }
 
@@ -2038,7 +2307,12 @@ class baseSQL
      */
     public function from($table)
     {
-        $this->sql .= "FROM $table";
+        $this->sql         .= "FROM $table";
+        $this->currentTable = $table;
+
+        /* Default table. */
+        $this->tableAlias[''] = $table;
+
         return $this;
     }
 
@@ -2054,6 +2328,9 @@ class baseSQL
     {
         if($this->inCondition and !$this->conditionIsTrue) return $this;
         $this->sql .= " AS $alias ";
+
+        $this->tableAlias[$alias] = $this->currentTable;
+
         return $this;
     }
 
@@ -2068,7 +2345,9 @@ class baseSQL
     public function leftJoin($table)
     {
         if($this->inCondition and !$this->conditionIsTrue) return $this;
-        $this->sql .= " LEFT JOIN $table";
+        $this->sql         .= " LEFT JOIN $table";
+        $this->currentTable = $table;
+
         return $this;
     }
 
