@@ -5,9 +5,15 @@ require_once dirname(__DIR__) . DS . 'btn' . DS . 'v1.php';
 require_once dirname(__DIR__) . DS . 'sidebar' . DS . 'v1.php';
 class thinkStepMenu extends wg
 {
-    private array $modules = array();
+    /**
+     * @var mixed[]
+     */
+    private $modules = array();
 
-    protected static array $defineProps = array(
+    /**
+     * @var mixed[]
+     */
+    protected static $defineProps = array(
         'modules: array',
         'wizard: object',
         'marketID?: int',
@@ -36,22 +42,32 @@ class thinkStepMenu extends wg
         return file_get_contents(__DIR__ . DS . 'js' . DS . 'v1.js');
     }
 
-    private function getQuotedText($modules, $quotedTitle)
+    /**
+     * @param mixed[] $modules
+     * @param mixed[] $quotedTitle
+     * @param string $quoteIndex
+     */
+    private function getQuotedText($modules, $quotedTitle, &$quoteIndex = '')
     {
         foreach($modules as $item)
         {
-            if($item->id == $quotedTitle) return sprintf($this->lang->thinkstep->treeLabel, $item->index);
+            if(in_array($item->id, $quotedTitle)) $quoteIndex = $quoteIndex . (!empty($quoteIndex) ? '、' : '') . $item->index;
             if(!empty($item->children))
             {
-                $childrenResult = $this->getQuotedText($item->children, $quotedTitle);
+                $childrenResult = $this->getQuotedText($item->children, $quotedTitle, $quoteIndex);
                 if($childrenResult) return $childrenResult;
             }
         }
-        return '';
+        return $quoteIndex;
     }
 
+    /**
+     * @param mixed[] $items
+     * @param int $parentID
+     */
     private function buildMenuTree($items, $parentID = 0)
     {
+        global $config;
         if(empty($items)) $items = $this->modules;
         if(empty($items)) return array();
 
@@ -59,25 +75,45 @@ class thinkStepMenu extends wg
         $toggleNonNodeShow = $this->prop('toggleNonNodeShow');
         $hidden            = $this->prop('hidden');
         $sortTree          = $this->prop('sortable') || $this->prop('onSort');
+        $wizard            = $this->prop('wizard');
+        $hiddenModelType   = in_array($wizard->model, $config->thinkwizard->hiddenMenuModel);
         $parentItems       = array();
+        $questionItems     = array();
+
+        if($hiddenModelType) $questionItems = array_values($items);
+
         foreach($items as $setting)
         {
             if(!is_object($setting)) continue;
             $options     = !empty($setting->options) && is_string($setting->options) ? json_decode($setting->options) : array();
-            $quotedTitle = !empty($options->quoteTitle) ? $options->quoteTitle : null;
+            $quotedTitle = !empty($options->quoteTitle) ? explode(',', $options->quoteTitle) : null;
             $quotedText  = '';
             /* 给引用其他问题的多选题添加标签。Add tags to multiple-choice questions that reference other questions. */
             if($quotedTitle)
             {
-                $quotedText = $this->getQuotedText($this->modules, $quotedTitle);
+                $quotedIndex = $this->getQuotedText($this->modules, $quotedTitle);
+                $quotedText  = sprintf($this->lang->thinkstep->treeLabel, $quotedIndex);
             }
 
-            $canView     = common::hasPriv('thinkstep', 'view');
-            $unClickable = $toggleNonNodeShow && $setting->id != $activeKey && $setting->type != 'node' && json_decode($setting->answer) == null;
-            $item        = array(
+            $itemQuestionIndex = 0;
+
+            foreach ($questionItems as $index => $questionItem)
+            {
+                if($questionItem->id == $setting->id) $itemQuestionIndex = $index;
+            }
+
+            if(!empty($itemQuestionIndex) && $hiddenModelType) $preStep = $questionItems[$itemQuestionIndex - 1];
+
+            $canView       = common::hasPriv('thinkstep', 'view');
+            $unClickable   = $toggleNonNodeShow && $setting->id != $activeKey && $setting->type != 'node' && json_decode($setting->answer) == null;
+            $preStepAnswer = !empty($preStep) && !empty($preStep->answer) ? json_decode($preStep->answer, true) : array();
+            $preStepResult = !empty($preStepAnswer) ? $preStepAnswer['result'] : array();
+            $preHasAnswer  = (!empty($preStep) && $preStep->type == 'transition') ?  $preStepAnswer : $preStepResult;
+            $hiddenStep    = $unClickable || (!empty($preStep) && empty($preHasAnswer));
+            $item          = array(
                 'key'         => $setting->id,
                 'text'        => (isset($setting->index) ? ($setting->index . '. ') : '') . $setting->title,
-                'subtitle'    => !empty($quotedText) ? array('html' => "<span class='label size-sm rounded-full warning-pale'>$quotedText</span>") : null,
+                'subtitle'    => (!empty($quotedText) && !in_array($wizard->model, $config->thinkwizard->hiddenMenuModel) && $this->prop('showAction')) ? array('html' => "<span class='label size-sm rounded-full warning-pale'>$quotedText</span>") : null,
                 'hint'        => $unClickable ? $this->lang->thinkrun->error->unanswered :$setting->title,
                 'url'         => $unClickable || !$canView ? '' : $setting->url,
                 'data-id'     => $setting->id,
@@ -89,7 +125,7 @@ class thinkStepMenu extends wg
                 'disabled'    => $unClickable,
                 'actions'     => $this->prop('showAction') ? $this->getActions($setting) : null,
                 'data-wizard' => $setting->wizard,
-                'class'       => $unClickable && $hidden ? 'hidden': ''
+                'class'       => $hiddenStep && $hidden ? 'hidden' : ''
             );
 
             if($sortTree) $item['trailingIcon'] = 'move muted cursor-move';
@@ -147,6 +183,11 @@ class thinkStepMenu extends wg
         $wizard             = $this->prop('wizard');
         $canAddChild        = true;
         $showQuestionOfNode = true;
+        $hiddenModelType    = in_array($wizard->model, $config->thinkwizard->hiddenMenuModel);
+        $previewCanActions  = !$hiddenModelType || ($hiddenModelType && $item->type == 'transition');
+        $from               = '';
+        if($hiddenModelType) $from = strtolower($wizard->type);
+
         if(!empty($item->children))
         {
             foreach($item->children as $child)
@@ -155,6 +196,7 @@ class thinkStepMenu extends wg
                 if($showQuestionOfNode && $child->type == 'node') $showQuestionOfNode = false;
             }
         }
+        if($hiddenModelType) $canAddChild = false;
         $canCreate   = common::hasPriv('thinkstep', 'create');
         $canEdit     = common::hasPriv('thinkstep', 'edit');
         $canDelete   = common::hasPriv('thinkstep', 'delete');
@@ -165,7 +207,7 @@ class thinkStepMenu extends wg
         $transitionAction = array();
         if($canCreate)
         {
-            if($item->type == 'node') $menus[] = array(
+            if($item->type == 'node' && !$hiddenModelType) $menus[] = array(
                 'key'     => 'addNode',
                 'icon'    => 'add-chapter',
                 'text'    => $this->lang->thinkstep->actions['sameNode'],
@@ -178,7 +220,7 @@ class thinkStepMenu extends wg
                 'onClick' => jsRaw("() => addNode({$item->id}, 'child')")
             );
 
-            $transitionAction[] = array('type' => 'divider');
+            $transitionAction[] = $previewCanActions ? array('type' => 'divider') : null;
             $transitionAction[] = array(
                 'key'     => 'transition',
                 'icon'    => 'transition',
@@ -194,7 +236,7 @@ class thinkStepMenu extends wg
             'icon'         => 'trash',
             'text'         => $this->lang->thinkstep->actions['delete'],
             'innerClass'   => 'ajax-submit',
-            'data-url'     => createLink('thinkstep', 'delete', "marketID={$marketID}&stepID={$item->id}"),
+            'data-url'     => createLink('thinkstep', 'delete', "marketID={$marketID}&stepID={$item->id}&from={$from}"),
             'data-confirm' => $confirmTips,
         ) : array(
             'key'        => 'deleteNode',
@@ -218,19 +260,19 @@ class thinkStepMenu extends wg
             'innerClass'   => 'text-gray opacity-50',
             'hint'         => $this->lang->thinkstep->tips->linkBlocks
         );
-
         $menus = array_merge($menus, array(
-            $canEdit ? array(
+            ($canEdit && $previewCanActions) ? array(
                 'key'  => 'editNode',
                 'icon' => 'edit',
                 'text' => $this->lang->thinkstep->actions['edit'],
-                'url'  => createLink('thinkstep', 'edit', "marketID={$marketID}&stepID={$item->id}")
+                'url'  => createLink('thinkstep', 'edit', "marketID={$marketID}&stepID={$item->id}&from={$from}")
             ) : null,
-            $canDelete ? $deleteItem : null,
+            ($canDelete && $previewCanActions) ? $deleteItem : null,
             in_array($wizard->model, $config->thinkwizard->venn) && $item->type == 'question' && $canLink ? $linkItem : null
         ), $transitionAction);
 
-        if($canCreate && (($showQuestionOfNode && $item->type == 'node') || $item->hasSameQuestion || $item->type == 'question')) $menus = array_merge($menus, array(
+        $canAddQuestions = ($showQuestionOfNode && $item->type == 'node') || $item->hasSameQuestion || $item->type == 'question';
+        if($canCreate && !$hiddenModelType && $canAddQuestions) $menus = array_merge($menus, array(
             array('type' => 'divider'),
             $this->buildMenuItem('radio', 'radio', $this->lang->thinkstep->createStep . $this->lang->thinkstep->actions['radio'], $item, $parentID, 'radio'),
             $this->buildMenuItem('checkbox', 'checkbox', $this->lang->thinkstep->createStep . $this->lang->thinkstep->actions['checkbox'], $item, $parentID, 'checkbox'),
@@ -241,6 +283,13 @@ class thinkStepMenu extends wg
         return $menus;
     }
 
+    /**
+     * @param string $key
+     * @param string $icon
+     * @param string $text
+     * @param object $item
+     * @param int $parentID
+     */
     private function buildMenuItem($key, $icon, $text, $item, $parentID)
     {
         return array(
@@ -253,13 +302,7 @@ class thinkStepMenu extends wg
 
     private function buildActions()
     {
-        return btn
-        (
-            set::type('ghost'),
-            setClass('text-gray absolute top-2 right-3 z-10 toggle-btn'),
-            set::icon('fold-all'),
-            on::click('toggleQuestionShow'),
-        );
+        return btn(set::type('ghost'), setClass('text-gray absolute top-2 right-3 z-10 toggle-btn'), set::icon('fold-all'), on::click('toggleQuestionShow'));
     }
 
     protected function build()
