@@ -114,6 +114,16 @@ $lang->misc->feature->promptExecImage   = 'theme/default/images/main/prompt_exec
 $lang->misc->feature->promptLearnMore   = 'https://www.zentao.net/book/zentaopms/1097.html';
 
 /* Release Date. */
+$lang->misc->releaseDate['21.7']        = '2025-05-16';
+$lang->misc->releaseDate['21.6.1']      = '2025-04-30';
+$lang->misc->releaseDate['21.6']        = '2025-04-11';
+$lang->misc->releaseDate['21.6.beta']   = '2025-03-21';
+$lang->misc->releaseDate['21.5']        = '2025-03-06';
+$lang->misc->releaseDate['21.4']        = '2025-01-15';
+$lang->misc->releaseDate['21.3']        = '2024-12-27';
+$lang->misc->releaseDate['21.2']        = '2024-12-03';
+$lang->misc->releaseDate['21.1']        = '2024-11-15';
+$lang->misc->releaseDate['21.0']        = '2024-11-01';
 $lang->misc->releaseDate['20.8']        = '2024-10-21';
 $lang->misc->releaseDate['20.7.1']      = '2024-09-30';
 $lang->misc->releaseDate['20.7']        = '2024-09-14';
@@ -238,6 +248,16 @@ $lang->misc->releaseDate['7.1.stable']  = '2015-03-07';
 $lang->misc->releaseDate['6.3.stable']  = '2014-11-07';
 
 /* Release Detail. */
+$lang->misc->feature->all['21.7'][]        = array('title' => '', 'desc' => '');
+$lang->misc->feature->all['21.6.1'][]      = array('title' => '修復文檔相關Bug。', 'desc' => '');
+$lang->misc->feature->all['21.6'][]        = array('title' => 'Jira導入優化，文檔多人協作。', 'desc' => '');
+$lang->misc->feature->all['21.6.beta'][]   = array('title' => 'Jira導入2.0版本，Confluence導入。', 'desc' => '');
+$lang->misc->feature->all['21.5'][]        = array('title' => '性能優化，備註支持上傳附件性能優化，文檔優化。', 'desc' => '');
+$lang->misc->feature->all['21.4'][]        = array('title' => '測試相關細節優化，工作流細節優化，重構機會模組。', 'desc' => '');
+$lang->misc->feature->all['21.3'][]        = array('title' => '項目集、項目、執行列表中增加已延期的篩選標籤，項目創建成功後增加下一步工作的引導，瀑布項目中已有任務和其他數據的階段可以拆分子階段，工作流新增的欄位組件中支持設置提示信息，重構創建工單頁面，敏捷項目中增加機會功能，在不啟用迭代功能的項目中增加問題、風險、機會、過程、QA、會議功能。', 'desc' => '');
+$lang->misc->feature->all['21.2'][]        = array('title' => '發佈下增加應用，文檔下拉菜單增加項目集分組顯示和文檔權限提示，複製任務、需求、Bug、用例可以帶入附件，系統管理員可以刪除公共聯繫人數據，項目執行列表中增加任務搜索功能，執行列表中增加已延期篩選標籤，立項支持工作流擴展，BI增加版本管理，文檔編輯器兼容優化，反饋增加複製功能，創建反饋頁面所屬模組為空時可以維護模組，問題、風險、機會、審計增加消息通知，地盤貢獻增加貢獻的審計、基線列表，評審、審計詳情頁面細節優，甘特圖頁面支持任務搜索功能，設計增加需求變更後確認功能，評審報告、基線狀態報告支持導出。', 'desc' => '');
+$lang->misc->feature->all['21.1'][]        = array('title' => '文檔介面空間優化升級，介面空間核心功能入口優化，主機功能優化，對象間關聯關係完善，度量項補0邏輯，DuckDB相關優化，上線功能優化。', 'desc' => '');
+$lang->misc->feature->all['21.0'][]        = array('title' => '文檔功能優化；產品流程模板、項目流程模板BI設計器優化。', 'desc' => '');
 $lang->misc->feature->all['20.8'][]        = array('title' => '文檔優化：優化任務關係，修復Bug。', 'desc' => '');
 $lang->misc->feature->all['20.7.1'][]      = array('title' => '修復已知Bug。', 'desc' => '');
 $lang->misc->feature->all['20.7'][]        = array('title' => '新手引導優化；菜單自定義；工作流優化；OR界面增加貢獻模組。', 'desc' => '');
